"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.props = props;
exports.Node = exports.HEAD = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

function attrEscape(str) {
  return str.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/"/g, '&quot;').replace(/\t/g, '&#x9;').replace(/\n/g, '&#xA;').replace(/\r/g, '&#xD;');
}

function escape(str) {
  return str.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/\r/g, '&#xD;');
}

var HEAD = Symbol('head');
exports.HEAD = HEAD;

function props() {
  for (var _len = arguments.length, keys = new Array(_len), _key = 0; _key < _len; _key++) {
    keys[_key] = arguments[_key];
  }

  return function (target) {
    var _loop = function _loop() {
      var key = keys[_i];
      target.elements.push({
        key,
        kind: 'method',
        placement: 'prototype',
        descriptor: {
          get() {
            if (this.attributes) {
              return this.attributes[key];
            }
          },

          set(value) {
            if (this.attributes === undefined) {
              this.attributes = {};
            }

            this.attributes[key] = value;
          },

          configurable: true,
          enumerable: true
        }
      });
    };

    for (var _i = 0; _i < keys.length; _i++) {
      _loop();
    }

    return target;
  };
}

var Node =
/*#__PURE__*/
function () {
  function Node() {
    var attributes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var children = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var xmlName = arguments.length > 2 ? arguments[2] : undefined;
    (0, _classCallCheck2.default)(this, Node);

    var _arr = Object.keys(attributes);

    for (var _i2 = 0; _i2 < _arr.length; _i2++) {
      var key = _arr[_i2];
      this[key] = attributes[key];
    }

    this.children = children;
    this._xmlName = xmlName || this.constructor.name.substring(1);
  }

  (0, _createClass2.default)(Node, [{
    key: "render",
    value: function render() {
      function walk(tree) {
        var name = tree._xmlName;
        var attributes = tree.attributes,
            children = tree.children;
        var tokens = [];

        if (tree[HEAD]) {
          tokens.push(tree[HEAD]);
        }

        tokens.push(`<${name}`);

        var _arr2 = Object.keys(attributes || {});

        for (var _i3 = 0; _i3 < _arr2.length; _i3++) {
          var key = _arr2[_i3];
          var v = attributes[key];
          if (v === undefined) continue;

          if (typeof v === 'string') {
            v = attrEscape(v);
          }

          if (typeof v === 'boolean') {
            v = v ? 1 : 0;
          }

          tokens.push(` ${key}="${v}"`);
        }

        if (!children.length) {
          tokens.push('/>');
          return tokens;
        }

        tokens.push('>');
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = children[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var child = _step.value;

            if (child instanceof Node) {
              tokens.push(child.render());
            } else if (typeof child === 'string') {
              tokens.push(escape(child));
            } else {
              tokens.push(child.toString());
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        tokens.push(`</${name}>`);
        return tokens;
      }

      return walk(this).join('');
    }
  }]);
  return Node;
}();

exports.Node = Node;